import functools
import json

from flask import Blueprint, request, Response, current_app
from inventory_provider import alarmsdb, db

routes = Blueprint("inventory-alarmsdb-query-routes", __name__)


def require_accepts_json(f):
    """
    used as a route handler decorator to return an error
    unless the request allows responses with type "application/json"
    :param f: the function to be decorated
    :return: the decorated function
    """
    @functools.wraps(f)
    def decorated_function(*args, **kwargs):
        # TODO: use best_match to disallow */* ...?
        if not request.accept_mimetypes.accept_json:
            return Response(
                response="response will be json",
                status=406,
                mimetype="text/html")
        return f(*args, **kwargs)
    return decorated_function


@routes.route("/interface-status", methods=['GET', 'POST'])
@require_accepts_json
def get_interface_status():
    config = current_app.config['INVENTORY_PROVIDER_CONFIG']

    equipment = request.args.get("equipment")
    interface = request.args.get("interface")

    with db.connection(config['alarms-db']) as connection:
        result = {"status": alarmsdb.get_last_known_interface_status(
            connection, equipment, interface)}

    return Response(
        json.dumps(result),
        mimetype="application/json")
