import json
from collections import defaultdict
from flask import current_app
from inventory_provider import alarmsdb, db


services_key = "inv_services"
interfaces_key = "inv_interfaces"
equipment_locations_key = "inv_eq_locations"
service_child_to_parents_key = "inv_service_child_to_parents"
service_parent_to_children_key = "inv_service_parent_to_children"
interface_status_key = "service_status"


def update_services_to_monitor(services):
    r = db.get_redis()
    relevant_types = ("path", "service", "l2circuit")
    r.delete(services_key)
    for service in services:
        if service["circuit_type"].lower() in relevant_types:
            r.hset(services_key, service["id"], json.dumps(service))


def update_interfaces_to_services(services):
    r = db.get_redis()
    config = current_app.config["INVENTORY_PROVIDER_CONFIG"]
    with db.connection(config["alarms-db"]) as cnx:
        with db.cursor(cnx) as csr:
            mapped_interfaces = defaultdict(list)
            r.delete(interfaces_key)
            for service in services:
                key = "{}::{}".format(
                    service["equipment"],
                    service["interface_name"]
                )
                mapped_interfaces[key].append(service)
                if not r.hexists(interface_status_key, key):
                    r.hset(interface_status_key, key,
                           alarmsdb.get_last_known_interface_status(
                               csr,
                               service["equipment"],
                               service["interface_name"]
                           ))

    for key, value in mapped_interfaces.items():
        r.hset(interfaces_key, key, json.dumps(value))


def update_service_hierarchy(records):
    r = db.get_redis()
    children_to_parents = defaultdict(list)
    parents_to_children = defaultdict(list)
    for relation in records:
        parent_id = relation["parent_circuit_id"]
        child_id = relation["child_circuit_id"]
        parents_to_children[parent_id].append(relation)
        children_to_parents[child_id].append(relation)

    r.delete(service_child_to_parents_key)
    for child, parents in children_to_parents.items():
        r.hset(service_child_to_parents_key, child, json.dumps(parents))

    r.delete(service_parent_to_children_key)
    for parent, children in parents_to_children.items():
        r.hset(service_parent_to_children_key, parent, json.dumps(children))


def update_equipment_locations(equipment_location_data):
    r = db.get_redis()
    r.delete(equipment_locations_key)
    for ld in equipment_location_data:
        r.hset(equipment_locations_key, ld["equipment_name"], json.dumps(ld))
